# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .resource_availability import ResourceAvailability
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ClientFacingProviderWithStatus(UniversalBaseModel):
    name: str = pydantic.Field()
    """
    Name of source of information
    """

    slug: str = pydantic.Field()
    """
    Slug for designated source
    """

    logo: str = pydantic.Field()
    """
    URL for source logo
    """

    status: str = pydantic.Field()
    """
    Status of source, either error or connected
    """

    resource_availability: typing.Dict[str, ResourceAvailability]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import pydantic
from .sleep_type import SleepType
import typing
from .sleep_summary_state import SleepSummaryState
from .client_facing_source import ClientFacingSource
from .client_facing_sleep_stream import ClientFacingSleepStream
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class ClientFacingSleep(UniversalBaseModel):
    user_id: str = pydantic.Field()
    """
    User id returned by vital create user request. This id should be stored in your database against the user and used for all interactions with the vital api.
    """

    id: str
    date: str = pydantic.Field()
    """
    Date of the specified record, formatted as ISO8601 datetime string in UTC 00:00. Deprecated in favour of calendar_date.
    """

    calendar_date: str = pydantic.Field()
    """
    Date of the sleep summary in the YYYY-mm-dd format. This generally matches the sleep end date.
    """

    bedtime_start: str = pydantic.Field()
    """
    UTC Time when the sleep period started
    """

    bedtime_stop: str = pydantic.Field()
    """
    UTC Time when the sleep period ended
    """

    type: SleepType = pydantic.Field()
    """
    `long_sleep`: >=3 hours of sleep;
    `short_sleep`: <3 hours of sleep;
    `acknowledged_nap`: User-acknowledged naps, typically under 3 hours of sleep;
    `unknown`: The sleep session recording is ongoing.
    """

    timezone_offset: typing.Optional[int] = pydantic.Field(default=None)
    """
    Timezone offset from UTC as seconds. For example, EEST (Eastern European Summer Time, +3h) is 10800. PST (Pacific Standard Time, -8h) is -28800::seconds
    """

    duration: int = pydantic.Field()
    """
    Total duration of the sleep period (sleep.duration = sleep.bedtime_end - sleep.bedtime_start)::seconds
    """

    total: int = pydantic.Field()
    """
    Total amount of sleep registered during the sleep period (sleep.total = sleep.rem + sleep.light + sleep.deep)::seconds
    """

    awake: int = pydantic.Field()
    """
    Total amount of awake time registered during the sleep period::seconds
    """

    light: int = pydantic.Field()
    """
    Total amount of light sleep registered during the sleep period::seconds
    """

    rem: int = pydantic.Field()
    """
    Total amount of REM sleep registered during the sleep period, minutes::seconds
    """

    deep: int = pydantic.Field()
    """
    Total amount of deep (N3) sleep registered during the sleep period::seconds
    """

    score: typing.Optional[int] = pydantic.Field(default=None)
    """
    A value between 1 and 100 representing how well the user slept. Currently only available for Withings, Oura, Whoop and Garmin::scalar
    """

    hr_lowest: typing.Optional[int] = pydantic.Field(default=None)
    """
    The lowest heart rate (5 minutes sliding average) registered during the sleep period::beats per minute
    """

    hr_average: typing.Optional[int] = pydantic.Field(default=None)
    """
    The average heart rate registered during the sleep period::beats per minute
    """

    efficiency: typing.Optional[float] = pydantic.Field(default=None)
    """
    Sleep efficiency is the percentage of the sleep period spent asleep (100% \* sleep.total / sleep.duration)::perc
    """

    latency: typing.Optional[int] = pydantic.Field(default=None)
    """
    Detected latency from bedtime_start to the beginning of the first five minutes of persistent sleep::seconds
    """

    temperature_delta: typing.Optional[float] = pydantic.Field(default=None)
    """
    Skin temperature deviation from the long-term temperature average::celcius
    """

    skin_temperature: typing.Optional[float] = pydantic.Field(default=None)
    """
    The skin temperature::celcius
    """

    hr_dip: typing.Optional[float] = pydantic.Field(default=None)
    """
    Sleeping Heart Rate Dip is the percentage difference between your average waking heart rate and your average sleeping heart rate. In health studies, a greater "dip" is typically seen as a positive indicator of overall health. Currently only available for Garmin::perc
    """

    state: typing.Optional[SleepSummaryState] = pydantic.Field(default=None)
    """
    Some providers can provide updates to the sleep summary hours after the sleep period has ended. This field indicates the state of the sleep summary. For example, TENTATIVE means the summary is an intial prediction from the provider and can be subject to change. Currently only available for Garmin and EightSleep::str
    """

    average_hrv: typing.Optional[float] = pydantic.Field(default=None)
    """
    The average heart rate variability registered during the sleep period::rmssd
    """

    respiratory_rate: typing.Optional[float] = pydantic.Field(default=None)
    """
    Average respiratory rate::breaths per minute
    """

    source: ClientFacingSource = pydantic.Field()
    """
    Source the data has come from.
    """

    sleep_stream: typing.Optional[ClientFacingSleepStream] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.paginated_users_response import PaginatedUsersResponse
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.client_facing_user_key import ClientFacingUserKey
from ..errors.bad_request_error import BadRequestError
from ..types.metrics_result import MetricsResult
from ..types.user_sign_in_token_response import UserSignInTokenResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..types.client_facing_provider_with_status import ClientFacingProviderWithStatus
from ..types.client_facing_user import ClientFacingUser
from ..types.user_success_response import UserSuccessResponse
from ..types.user_info import UserInfo
from ..types.responsible_relationship import ResponsibleRelationship
from ..types.vital_core_schemas_db_schemas_lab_test_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
)
from ..types.guarantor_details import GuarantorDetails
from ..types.client_facing_insurance import ClientFacingInsurance
from ..types.address import Address
from ..types.providers import Providers
from ..types.user_refresh_success_response import UserRefreshSuccessResponse
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters
        ----------
        offset : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedUsersResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_all()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/user",
            method="GET",
            params={
                "offset": offset,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PaginatedUsersResponse,
                    parse_obj_as(
                        type_=PaginatedUsersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        client_user_id: str,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user data ingestion. Data older than this date will not be ingested.

        ingestion_end : typing.Optional[str]
            Ending bound for user data ingestion. Data from this date or later will not be ingested and the connection will be deregistered.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUserKey
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.create(
            client_user_id="client_user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/user",
            method="POST",
            json={
                "client_user_id": client_user_id,
                "fallback_time_zone": fallback_time_zone,
                "fallback_birth_date": fallback_birth_date,
                "ingestion_start": ingestion_start,
                "ingestion_end": ingestion_end,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUserKey,
                    parse_obj_as(
                        type_=ClientFacingUserKey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_team_metrics(self, *, request_options: typing.Optional[RequestOptions] = None) -> MetricsResult:
        """
        GET metrics for team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsResult
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_team_metrics()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/user/metrics",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    MetricsResult,
                    parse_obj_as(
                        type_=MetricsResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_user_sign_in_token(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSignInTokenResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSignInTokenResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_user_sign_in_token(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/sign_in_token",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSignInTokenResponse,
                    parse_obj_as(
                        type_=UserSignInTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_connected_providers(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_connected_providers(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/providers/{jsonable_encoder(user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.List[ClientFacingProviderWithStatus]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.List[ClientFacingProviderWithStatus]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ClientFacingUser:
        """
        GET User details given the user_id.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUser,
                    parse_obj_as(
                        type_=ClientFacingUser,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.delete(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def patch(
        self,
        user_id: str,
        *,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        client_user_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        user_id : str

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user data ingestion. Data older than this date will not be ingested.

        ingestion_end : typing.Optional[str]
            Ending bound for user data ingestion. Data from this date or later will not be ingested and the connection will be deregistered.

        client_user_id : typing.Optional[str]
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.patch(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="PATCH",
            json={
                "fallback_time_zone": fallback_time_zone,
                "fallback_birth_date": fallback_birth_date,
                "ingestion_start": ingestion_start,
                "ingestion_end": ingestion_end,
                "client_user_id": client_user_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_latest_user_info(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_latest_user_info(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/info/latest",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserInfo,
                    parse_obj_as(
                        type_=UserInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_insurance(
        self,
        user_id: str,
        *,
        payor_code: str,
        member_id: str,
        relationship: ResponsibleRelationship,
        insured: VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        group_id: typing.Optional[str] = OMIT,
        guarantor: typing.Optional[GuarantorDetails] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        payor_code : str

        member_id : str

        relationship : ResponsibleRelationship

        insured : VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails

        group_id : typing.Optional[str]

        guarantor : typing.Optional[GuarantorDetails]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import (
            Address,
            Gender,
            ResponsibleRelationship,
            Vital,
            VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        )

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.create_insurance(
            user_id="user_id",
            payor_code="payor_code",
            member_id="member_id",
            relationship=ResponsibleRelationship.SELF,
            insured=VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails(
                first_name="first_name",
                last_name="last_name",
                gender=Gender.FEMALE,
                address=Address(
                    first_line="first_line",
                    country="country",
                    zip="zip",
                    city="city",
                    state="state",
                ),
                dob="dob",
                email="email",
                phone_number="phone_number",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/insurance",
            method="POST",
            json={
                "payor_code": payor_code,
                "member_id": member_id,
                "group_id": group_id,
                "relationship": relationship,
                "insured": insured,
                "guarantor": guarantor,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingInsurance,
                    parse_obj_as(
                        type_=ClientFacingInsurance,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_latest_insurance(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_latest_insurance(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/insurance/latest",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingInsurance,
                    parse_obj_as(
                        type_=ClientFacingInsurance,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def upsert_user_info(
        self,
        user_id: str,
        *,
        first_name: str,
        last_name: str,
        email: str,
        phone_number: str,
        gender: str,
        dob: str,
        address: Address,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        first_name : str

        last_name : str

        email : str

        phone_number : str

        gender : str

        dob : str

        address : Address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        from vital import Address, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.upsert_user_info(
            user_id="user_id",
            first_name="first_name",
            last_name="last_name",
            email="email",
            phone_number="phone_number",
            gender="gender",
            dob="dob",
            address=Address(
                first_line="first_line",
                country="country",
                zip="zip",
                city="city",
                state="state",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/info",
            method="PATCH",
            json={
                "first_name": first_name,
                "last_name": last_name,
                "email": email,
                "phone_number": phone_number,
                "gender": gender,
                "dob": dob,
                "address": address,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserInfo,
                    parse_obj_as(
                        type_=UserInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_by_client_user_id(
        self, client_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.get_by_client_user_id(
            client_user_id="client_user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/resolve/{jsonable_encoder(client_user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUser,
                    parse_obj_as(
                        type_=ClientFacingUser,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def deregister_provider(
        self, user_id: str, provider: Providers, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        provider : Providers
            Provider slug. e.g., `oura`, `fitbit`, `garmin`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Providers, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.deregister_provider(
            user_id="user_id",
            provider=Providers.OURA,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/{jsonable_encoder(provider)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def undo_delete(
        self,
        *,
        user_id: typing.Optional[str] = None,
        client_user_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : typing.Optional[str]
            User ID to undo deletion. Mutually exclusive with `client_user_id`.

        client_user_id : typing.Optional[str]
            Client User ID to undo deletion. Mutually exclusive with `user_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.undo_delete()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/user/undo_delete",
            method="POST",
            params={
                "user_id": user_id,
                "client_user_id": client_user_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def refresh(
        self,
        user_id: str,
        *,
        timeout: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters
        ----------
        user_id : str

        timeout : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserRefreshSuccessResponse
            Successful Response

        Examples
        --------
        from vital import Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.user.refresh(
            user_id="user_id",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/user/refresh/{jsonable_encoder(user_id)}",
            method="POST",
            params={
                "timeout": timeout,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserRefreshSuccessResponse,
                    parse_obj_as(
                        type_=UserRefreshSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all(
        self,
        *,
        offset: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PaginatedUsersResponse:
        """
        GET All users for team.

        Parameters
        ----------
        offset : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PaginatedUsersResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_all()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/user",
            method="GET",
            params={
                "offset": offset,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PaginatedUsersResponse,
                    parse_obj_as(
                        type_=PaginatedUsersResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        client_user_id: str,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingUserKey:
        """
        POST Create a Vital user given a client_user_id and returns the user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user data ingestion. Data older than this date will not be ingested.

        ingestion_end : typing.Optional[str]
            Ending bound for user data ingestion. Data from this date or later will not be ingested and the connection will be deregistered.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUserKey
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.create(
                client_user_id="client_user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/user",
            method="POST",
            json={
                "client_user_id": client_user_id,
                "fallback_time_zone": fallback_time_zone,
                "fallback_birth_date": fallback_birth_date,
                "ingestion_start": ingestion_start,
                "ingestion_end": ingestion_end,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUserKey,
                    parse_obj_as(
                        type_=ClientFacingUserKey,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_team_metrics(self, *, request_options: typing.Optional[RequestOptions] = None) -> MetricsResult:
        """
        GET metrics for team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        MetricsResult
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_team_metrics()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/user/metrics",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    MetricsResult,
                    parse_obj_as(
                        type_=MetricsResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_user_sign_in_token(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSignInTokenResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSignInTokenResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_user_sign_in_token(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/sign_in_token",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSignInTokenResponse,
                    parse_obj_as(
                        type_=UserSignInTokenResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_connected_providers(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]:
        """
        GET Users connected providers

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Dict[str, typing.List[ClientFacingProviderWithStatus]]
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_connected_providers(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/providers/{jsonable_encoder(user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.Dict[str, typing.List[ClientFacingProviderWithStatus]],
                    parse_obj_as(
                        type_=typing.Dict[str, typing.List[ClientFacingProviderWithStatus]],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ClientFacingUser:
        """
        GET User details given the user_id.

        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUser,
                    parse_obj_as(
                        type_=ClientFacingUser,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.delete(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def patch(
        self,
        user_id: str,
        *,
        fallback_time_zone: typing.Optional[str] = OMIT,
        fallback_birth_date: typing.Optional[str] = OMIT,
        ingestion_start: typing.Optional[str] = OMIT,
        ingestion_end: typing.Optional[str] = OMIT,
        client_user_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        user_id : str

        fallback_time_zone : typing.Optional[str]

                Fallback time zone of the user, in the form of a valid IANA tzdatabase identifier (e.g., `Europe/London` or `America/Los_Angeles`).
                Used when pulling data from sources that are completely time zone agnostic (e.g., all time is relative to UTC clock, without any time zone attributions on data points).


        fallback_birth_date : typing.Optional[str]
            Fallback date of birth of the user, in YYYY-mm-dd format. Used for calculating max heartrate for providers that don not provide users' age.

        ingestion_start : typing.Optional[str]
            Starting bound for user data ingestion. Data older than this date will not be ingested.

        ingestion_end : typing.Optional[str]
            Ending bound for user data ingestion. Data from this date or later will not be ingested and the connection will be deregistered.

        client_user_id : typing.Optional[str]
            A unique ID representing the end user. Typically this will be a user ID from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.patch(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}",
            method="PATCH",
            json={
                "fallback_time_zone": fallback_time_zone,
                "fallback_birth_date": fallback_birth_date,
                "ingestion_start": ingestion_start,
                "ingestion_end": ingestion_end,
                "client_user_id": client_user_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_latest_user_info(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_latest_user_info(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/info/latest",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserInfo,
                    parse_obj_as(
                        type_=UserInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_insurance(
        self,
        user_id: str,
        *,
        payor_code: str,
        member_id: str,
        relationship: ResponsibleRelationship,
        insured: VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        group_id: typing.Optional[str] = OMIT,
        guarantor: typing.Optional[GuarantorDetails] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        payor_code : str

        member_id : str

        relationship : ResponsibleRelationship

        insured : VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails

        group_id : typing.Optional[str]

        guarantor : typing.Optional[GuarantorDetails]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        import asyncio

        from vital import (
            Address,
            AsyncVital,
            Gender,
            ResponsibleRelationship,
            VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
        )

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.create_insurance(
                user_id="user_id",
                payor_code="payor_code",
                member_id="member_id",
                relationship=ResponsibleRelationship.SELF,
                insured=VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails(
                    first_name="first_name",
                    last_name="last_name",
                    gender=Gender.FEMALE,
                    address=Address(
                        first_line="first_line",
                        country="country",
                        zip="zip",
                        city="city",
                        state="state",
                    ),
                    dob="dob",
                    email="email",
                    phone_number="phone_number",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/insurance",
            method="POST",
            json={
                "payor_code": payor_code,
                "member_id": member_id,
                "group_id": group_id,
                "relationship": relationship,
                "insured": insured,
                "guarantor": guarantor,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingInsurance,
                    parse_obj_as(
                        type_=ClientFacingInsurance,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_latest_insurance(
        self, user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingInsurance:
        """
        Parameters
        ----------
        user_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingInsurance
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_latest_insurance(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/insurance/latest",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingInsurance,
                    parse_obj_as(
                        type_=ClientFacingInsurance,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def upsert_user_info(
        self,
        user_id: str,
        *,
        first_name: str,
        last_name: str,
        email: str,
        phone_number: str,
        gender: str,
        dob: str,
        address: Address,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserInfo:
        """
        Parameters
        ----------
        user_id : str

        first_name : str

        last_name : str

        email : str

        phone_number : str

        gender : str

        dob : str

        address : Address

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserInfo
            Successful Response

        Examples
        --------
        import asyncio

        from vital import Address, AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.upsert_user_info(
                user_id="user_id",
                first_name="first_name",
                last_name="last_name",
                email="email",
                phone_number="phone_number",
                gender="gender",
                dob="dob",
                address=Address(
                    first_line="first_line",
                    country="country",
                    zip="zip",
                    city="city",
                    state="state",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/info",
            method="PATCH",
            json={
                "first_name": first_name,
                "last_name": last_name,
                "email": email,
                "phone_number": phone_number,
                "gender": gender,
                "dob": dob,
                "address": address,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserInfo,
                    parse_obj_as(
                        type_=UserInfo,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_by_client_user_id(
        self, client_user_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ClientFacingUser:
        """
        GET user_id from client_user_id.

        Parameters
        ----------
        client_user_id : str
            A unique ID representing the end user. Typically this will be a user ID number from your application. Personally identifiable information, such as an email address or phone number, should not be used in the client_user_id.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ClientFacingUser
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.get_by_client_user_id(
                client_user_id="client_user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/resolve/{jsonable_encoder(client_user_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ClientFacingUser,
                    parse_obj_as(
                        type_=ClientFacingUser,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def deregister_provider(
        self, user_id: str, provider: Providers, *, request_options: typing.Optional[RequestOptions] = None
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : str

        provider : Providers
            Provider slug. e.g., `oura`, `fitbit`, `garmin`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, Providers

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.deregister_provider(
                user_id="user_id",
                provider=Providers.OURA,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/{jsonable_encoder(user_id)}/{jsonable_encoder(provider)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def undo_delete(
        self,
        *,
        user_id: typing.Optional[str] = None,
        client_user_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserSuccessResponse:
        """
        Parameters
        ----------
        user_id : typing.Optional[str]
            User ID to undo deletion. Mutually exclusive with `client_user_id`.

        client_user_id : typing.Optional[str]
            Client User ID to undo deletion. Mutually exclusive with `user_id`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.undo_delete()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/user/undo_delete",
            method="POST",
            params={
                "user_id": user_id,
                "client_user_id": client_user_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserSuccessResponse,
                    parse_obj_as(
                        type_=UserSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def refresh(
        self,
        user_id: str,
        *,
        timeout: typing.Optional[float] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UserRefreshSuccessResponse:
        """
        Trigger a manual refresh for a specific user

        Parameters
        ----------
        user_id : str

        timeout : typing.Optional[float]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UserRefreshSuccessResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.user.refresh(
                user_id="user_id",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/user/refresh/{jsonable_encoder(user_id)}",
            method="POST",
            params={
                "timeout": timeout,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UserRefreshSuccessResponse,
                    parse_obj_as(
                        type_=UserRefreshSuccessResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..types.patient_details import PatientDetails
from ..types.patient_address_compatible import PatientAddressCompatible
from ..types.physician_create_request_base import PhysicianCreateRequestBase
from ..types.health_insurance_create_request import HealthInsuranceCreateRequest
from ..types.consent import Consent
from ..core.request_options import RequestOptions
from ..types.post_order_response import PostOrderResponse
from ..core.pydantic_utilities import parse_obj_as
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.http_validation_error import HttpValidationError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.shipping_address import ShippingAddress
from ..core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TestkitClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register(
        self,
        *,
        user_id: str,
        sample_id: str,
        patient_details: PatientDetails,
        patient_address: PatientAddressCompatible,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        health_insurance: typing.Optional[HealthInsuranceCreateRequest] = OMIT,
        consents: typing.Optional[typing.Sequence[Consent]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Parameters
        ----------
        user_id : str

        sample_id : str

        patient_details : PatientDetails

        patient_address : PatientAddressCompatible

        physician : typing.Optional[PhysicianCreateRequestBase]

        health_insurance : typing.Optional[HealthInsuranceCreateRequest]

        consents : typing.Optional[typing.Sequence[Consent]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        import datetime

        from vital import Gender, PatientAddressCompatible, PatientDetails, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.testkit.register(
            user_id="user_id",
            sample_id="sample_id",
            patient_details=PatientDetails(
                first_name="first_name",
                last_name="last_name",
                dob=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                gender=Gender.FEMALE,
                phone_number="phone_number",
                email="email",
            ),
            patient_address=PatientAddressCompatible(
                first_line="first_line",
                city="city",
                state="state",
                zip="zip",
                country="country",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v3/order/testkit/register",
            method="POST",
            json={
                "user_id": user_id,
                "sample_id": sample_id,
                "patient_details": patient_details,
                "patient_address": patient_address,
                "physician": physician,
                "health_insurance": health_insurance,
                "consents": consents,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PostOrderResponse,
                    parse_obj_as(
                        type_=PostOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create_order(
        self,
        *,
        user_id: str,
        lab_test_id: str,
        shipping_details: ShippingAddress,
        passthrough: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters
        ----------
        user_id : str

        lab_test_id : str

        shipping_details : ShippingAddress

        passthrough : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        from vital import ShippingAddress, Vital

        client = Vital(
            api_key="YOUR_API_KEY",
        )
        client.testkit.create_order(
            user_id="user_id",
            lab_test_id="lab_test_id",
            shipping_details=ShippingAddress(
                receiver_name="receiver_name",
                first_line="first_line",
                city="city",
                state="state",
                zip="zip",
                country="country",
                phone_number="phone_number",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v3/order/testkit",
            method="POST",
            json={
                "user_id": user_id,
                "lab_test_id": lab_test_id,
                "shipping_details": shipping_details,
                "passthrough": passthrough,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PostOrderResponse,
                    parse_obj_as(
                        type_=PostOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncTestkitClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register(
        self,
        *,
        user_id: str,
        sample_id: str,
        patient_details: PatientDetails,
        patient_address: PatientAddressCompatible,
        physician: typing.Optional[PhysicianCreateRequestBase] = OMIT,
        health_insurance: typing.Optional[HealthInsuranceCreateRequest] = OMIT,
        consents: typing.Optional[typing.Sequence[Consent]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Parameters
        ----------
        user_id : str

        sample_id : str

        patient_details : PatientDetails

        patient_address : PatientAddressCompatible

        physician : typing.Optional[PhysicianCreateRequestBase]

        health_insurance : typing.Optional[HealthInsuranceCreateRequest]

        consents : typing.Optional[typing.Sequence[Consent]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from vital import AsyncVital, Gender, PatientAddressCompatible, PatientDetails

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.testkit.register(
                user_id="user_id",
                sample_id="sample_id",
                patient_details=PatientDetails(
                    first_name="first_name",
                    last_name="last_name",
                    dob=datetime.datetime.fromisoformat(
                        "2024-01-15 09:30:00+00:00",
                    ),
                    gender=Gender.FEMALE,
                    phone_number="phone_number",
                    email="email",
                ),
                patient_address=PatientAddressCompatible(
                    first_line="first_line",
                    city="city",
                    state="state",
                    zip="zip",
                    country="country",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v3/order/testkit/register",
            method="POST",
            json={
                "user_id": user_id,
                "sample_id": sample_id,
                "patient_details": patient_details,
                "patient_address": patient_address,
                "physician": physician,
                "health_insurance": health_insurance,
                "consents": consents,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PostOrderResponse,
                    parse_obj_as(
                        type_=PostOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create_order(
        self,
        *,
        user_id: str,
        lab_test_id: str,
        shipping_details: ShippingAddress,
        passthrough: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostOrderResponse:
        """
        Creates an order for an unregistered testkit

        Parameters
        ----------
        user_id : str

        lab_test_id : str

        shipping_details : ShippingAddress

        passthrough : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostOrderResponse
            Successful Response

        Examples
        --------
        import asyncio

        from vital import AsyncVital, ShippingAddress

        client = AsyncVital(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.testkit.create_order(
                user_id="user_id",
                lab_test_id="lab_test_id",
                shipping_details=ShippingAddress(
                    receiver_name="receiver_name",
                    first_line="first_line",
                    city="city",
                    state="state",
                    zip="zip",
                    country="country",
                    phone_number="phone_number",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v3/order/testkit",
            method="POST",
            json={
                "user_id": user_id,
                "lab_test_id": lab_test_id,
                "shipping_details": shipping_details,
                "passthrough": passthrough,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    PostOrderResponse,
                    parse_obj_as(
                        type_=PostOrderResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class UsAddress(UniversalBaseModel):
    first_line: str
    second_line: typing.Optional[str] = None
    city: str
    state: str
    zip_code: str
    unit: typing.Optional[str] = pydantic.Field(default=None)
    """
    Deprecated. Use `second_line` instead to provide the unit number. Subject to removal after 20 Nov 2023.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class PasswordProviders(str, enum.Enum):
    WHOOP = "whoop"
    RENPHO = "renpho"
    PELOTON = "peloton"
    ZWIFT = "zwift"
    EIGHT_SLEEP = "eight_sleep"
    BEURER_API = "beurer_api"
    DEXCOM = "dexcom"
    HAMMERHEAD = "hammerhead"
    MY_FITNESS_PAL = "my_fitness_pal"
    KARDIA = "kardia"
    ABBOTT_LIBREVIEW = "abbott_libreview"

    def visit(
        self,
        whoop: typing.Callable[[], T_Result],
        renpho: typing.Callable[[], T_Result],
        peloton: typing.Callable[[], T_Result],
        zwift: typing.Callable[[], T_Result],
        eight_sleep: typing.Callable[[], T_Result],
        beurer_api: typing.Callable[[], T_Result],
        dexcom: typing.Callable[[], T_Result],
        hammerhead: typing.Callable[[], T_Result],
        my_fitness_pal: typing.Callable[[], T_Result],
        kardia: typing.Callable[[], T_Result],
        abbott_libreview: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is PasswordProviders.WHOOP:
            return whoop()
        if self is PasswordProviders.RENPHO:
            return renpho()
        if self is PasswordProviders.PELOTON:
            return peloton()
        if self is PasswordProviders.ZWIFT:
            return zwift()
        if self is PasswordProviders.EIGHT_SLEEP:
            return eight_sleep()
        if self is PasswordProviders.BEURER_API:
            return beurer_api()
        if self is PasswordProviders.DEXCOM:
            return dexcom()
        if self is PasswordProviders.HAMMERHEAD:
            return hammerhead()
        if self is PasswordProviders.MY_FITNESS_PAL:
            return my_fitness_pal()
        if self is PasswordProviders.KARDIA:
            return kardia()
        if self is PasswordProviders.ABBOTT_LIBREVIEW:
            return abbott_libreview()



# This file was auto-generated by Fern from our API Definition.

from .types import LabTestsGetOrdersRequestOrderDirection, LabTestsGetOrdersRequestOrderKey

__all__ = ["LabTestsGetOrdersRequestOrderDirection", "LabTestsGetOrdersRequestOrderKey"]



# This file was auto-generated by Fern from our API Definition.

import typing
from .date_trunc_expr import DateTruncExpr
from .date_part_expr import DatePartExpr

QueryInstructionGroupByItem = typing.Union[DateTruncExpr, DatePartExpr]



# This file was auto-generated by Fern from our API Definition.

from .activity_column_expr import ActivityColumnExpr
from .activity_column_expr_activity import ActivityColumnExprActivity
from .activity_v_2_in_db import ActivityV2InDb
from .add_on_order import AddOnOrder
from .address import Address
from .aggregate_expr import AggregateExpr
from .aggregate_expr_arg import AggregateExprArg
from .aggregate_expr_func import AggregateExprFunc
from .allowed_radius import AllowedRadius
from .answer import Answer
from .ao_e import AoE
from .ao_e_answer import AoEAnswer
from .appointment_availability_slots import AppointmentAvailabilitySlots
from .appointment_booking_request import AppointmentBookingRequest
from .appointment_event_status import AppointmentEventStatus
from .appointment_location import AppointmentLocation
from .appointment_provider import AppointmentProvider
from .appointment_psc_labs import AppointmentPscLabs
from .appointment_reschedule_request import AppointmentRescheduleRequest
from .appointment_service_type import AppointmentServiceType
from .appointment_status import AppointmentStatus
from .appointment_type import AppointmentType
from .area_info import AreaInfo
from .attempt_status import AttemptStatus
from .auth_type import AuthType
from .availability import Availability
from .basal_body_temperature_entry import BasalBodyTemperatureEntry
from .billing import Billing
from .biomarker_result import BiomarkerResult
from .body_column_expr import BodyColumnExpr
from .body_column_expr_body import BodyColumnExprBody
from .body_v_2_in_db import BodyV2InDb
from .cervical_mucus_entry import CervicalMucusEntry
from .cervical_mucus_entry_quality import CervicalMucusEntryQuality
from .client_activity_response import ClientActivityResponse
from .client_body_response import ClientBodyResponse
from .client_facing_activity import ClientFacingActivity
from .client_facing_api_key import ClientFacingApiKey
from .client_facing_appointment import ClientFacingAppointment
from .client_facing_appointment_cancellation_reason import ClientFacingAppointmentCancellationReason
from .client_facing_appointment_event import ClientFacingAppointmentEvent
from .client_facing_at_home_phlebotomy_order import ClientFacingAtHomePhlebotomyOrder
from .client_facing_at_home_phlebotomy_order_details import ClientFacingAtHomePhlebotomyOrderDetails
from .client_facing_blood_oxygen_timeseries import ClientFacingBloodOxygenTimeseries
from .client_facing_blood_pressure_timeseries import ClientFacingBloodPressureTimeseries
from .client_facing_body import ClientFacingBody
from .client_facing_body_fat_timeseries import ClientFacingBodyFatTimeseries
from .client_facing_body_temperature_delta_sample import ClientFacingBodyTemperatureDeltaSample
from .client_facing_body_temperature_delta_sample_sensor_location import (
    ClientFacingBodyTemperatureDeltaSampleSensorLocation,
)
from .client_facing_body_temperature_sample import ClientFacingBodyTemperatureSample
from .client_facing_body_temperature_sample_sensor_location import ClientFacingBodyTemperatureSampleSensorLocation
from .client_facing_body_weight_timeseries import ClientFacingBodyWeightTimeseries
from .client_facing_caffeine_timeseries import ClientFacingCaffeineTimeseries
from .client_facing_calories_active_timeseries import ClientFacingCaloriesActiveTimeseries
from .client_facing_calories_basal_timeseries import ClientFacingCaloriesBasalTimeseries
from .client_facing_carbohydrates_sample import ClientFacingCarbohydratesSample
from .client_facing_cholesterol_timeseries import ClientFacingCholesterolTimeseries
from .client_facing_diagnosis_information import ClientFacingDiagnosisInformation
from .client_facing_distance_timeseries import ClientFacingDistanceTimeseries
from .client_facing_electrocardiogram_voltage_timeseries import ClientFacingElectrocardiogramVoltageTimeseries
from .client_facing_floors_climbed_timeseries import ClientFacingFloorsClimbedTimeseries
from .client_facing_food import ClientFacingFood
from .client_facing_glucose_timeseries import ClientFacingGlucoseTimeseries
from .client_facing_grouped_timeseries_response_client_facing_body_temperature_delta_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingBodyTemperatureDeltaSample,
)
from .client_facing_grouped_timeseries_response_client_facing_body_temperature_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingBodyTemperatureSample,
)
from .client_facing_grouped_timeseries_response_client_facing_carbohydrates_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingCarbohydratesSample,
)
from .client_facing_grouped_timeseries_response_client_facing_insulin_injection_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingInsulinInjectionSample,
)
from .client_facing_grouped_timeseries_response_client_facing_note_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingNoteSample,
)
from .client_facing_grouped_timeseries_response_client_facing_workout_duration_sample import (
    ClientFacingGroupedTimeseriesResponseClientFacingWorkoutDurationSample,
)
from .client_facing_heart_rate import ClientFacingHeartRate
from .client_facing_heart_rate_timeseries import ClientFacingHeartRateTimeseries
from .client_facing_hrv_timeseries import ClientFacingHrvTimeseries
from .client_facing_hypnogram_timeseries import ClientFacingHypnogramTimeseries
from .client_facing_ige_timeseries import ClientFacingIgeTimeseries
from .client_facing_igg_timeseries import ClientFacingIggTimeseries
from .client_facing_insulin_injection_sample import ClientFacingInsulinInjectionSample
from .client_facing_insulin_injection_sample_type import ClientFacingInsulinInjectionSampleType
from .client_facing_insurance import ClientFacingInsurance
from .client_facing_lab import ClientFacingLab
from .client_facing_lab_location import ClientFacingLabLocation
from .client_facing_lab_test import ClientFacingLabTest
from .client_facing_labs import ClientFacingLabs
from .client_facing_loinc import ClientFacingLoinc
from .client_facing_marker import ClientFacingMarker
from .client_facing_marker_complete import ClientFacingMarkerComplete
from .client_facing_meal_response import ClientFacingMealResponse
from .client_facing_mindfulness_minutes_timeseries import ClientFacingMindfulnessMinutesTimeseries
from .client_facing_note_sample import ClientFacingNoteSample
from .client_facing_note_sample_tags_item import ClientFacingNoteSampleTagsItem
from .client_facing_order import ClientFacingOrder
from .client_facing_order_details import (
    ClientFacingOrderDetails,
    ClientFacingOrderDetails_AtHomePhlebotomy,
    ClientFacingOrderDetails_Testkit,
    ClientFacingOrderDetails_WalkInTest,
)
from .client_facing_order_event import ClientFacingOrderEvent
from .client_facing_patient_details_compatible import ClientFacingPatientDetailsCompatible
from .client_facing_payor_search_response import ClientFacingPayorSearchResponse
from .client_facing_physician import ClientFacingPhysician
from .client_facing_profile import ClientFacingProfile
from .client_facing_provider import ClientFacingProvider
from .client_facing_provider_detailed import ClientFacingProviderDetailed
from .client_facing_provider_with_status import ClientFacingProviderWithStatus
from .client_facing_resource import ClientFacingResource
from .client_facing_respiratory_rate_timeseries import ClientFacingRespiratoryRateTimeseries
from .client_facing_result import ClientFacingResult
from .client_facing_sample_grouping_keys import ClientFacingSampleGroupingKeys
from .client_facing_shipment import ClientFacingShipment
from .client_facing_sleep import ClientFacingSleep
from .client_facing_sleep_stream import ClientFacingSleepStream
from .client_facing_source import ClientFacingSource
from .client_facing_sport import ClientFacingSport
from .client_facing_steps_timeseries import ClientFacingStepsTimeseries
from .client_facing_stream import ClientFacingStream
from .client_facing_stream_altitude import ClientFacingStreamAltitude
from .client_facing_stream_cadence import ClientFacingStreamCadence
from .client_facing_stream_distance import ClientFacingStreamDistance
from .client_facing_stream_heartrate import ClientFacingStreamHeartrate
from .client_facing_stream_lat import ClientFacingStreamLat
from .client_facing_stream_lng import ClientFacingStreamLng
from .client_facing_stream_power import ClientFacingStreamPower
from .client_facing_stream_resistance import ClientFacingStreamResistance
from .client_facing_stream_temperature import ClientFacingStreamTemperature
from .client_facing_stream_velocity_smooth import ClientFacingStreamVelocitySmooth
from .client_facing_stress_level_timeseries import ClientFacingStressLevelTimeseries
from .client_facing_team import ClientFacingTeam
from .client_facing_test_kit_order_details import ClientFacingTestKitOrderDetails
from .client_facing_testkit_order import ClientFacingTestkitOrder
from .client_facing_timeseries_group_client_facing_body_temperature_delta_sample import (
    ClientFacingTimeseriesGroupClientFacingBodyTemperatureDeltaSample,
)
from .client_facing_timeseries_group_client_facing_body_temperature_sample import (
    ClientFacingTimeseriesGroupClientFacingBodyTemperatureSample,
)
from .client_facing_timeseries_group_client_facing_carbohydrates_sample import (
    ClientFacingTimeseriesGroupClientFacingCarbohydratesSample,
)
from .client_facing_timeseries_group_client_facing_insulin_injection_sample import (
    ClientFacingTimeseriesGroupClientFacingInsulinInjectionSample,
)
from .client_facing_timeseries_group_client_facing_note_sample import ClientFacingTimeseriesGroupClientFacingNoteSample
from .client_facing_timeseries_group_client_facing_workout_duration_sample import (
    ClientFacingTimeseriesGroupClientFacingWorkoutDurationSample,
)
from .client_facing_user import ClientFacingUser
from .client_facing_user_key import ClientFacingUserKey
from .client_facing_vo_2_max_timeseries import ClientFacingVo2MaxTimeseries
from .client_facing_walk_in_order_details import ClientFacingWalkInOrderDetails
from .client_facing_walk_in_test_order import ClientFacingWalkInTestOrder
from .client_facing_water_timeseries import ClientFacingWaterTimeseries
from .client_facing_workout import ClientFacingWorkout
from .client_facing_workout_duration_sample import ClientFacingWorkoutDurationSample
from .client_facing_workout_duration_sample_intensity import ClientFacingWorkoutDurationSampleIntensity
from .client_sleep_cycle_response import ClientSleepCycleResponse
from .client_sleep_response import ClientSleepResponse
from .client_user_id_conflict import ClientUserIdConflict
from .client_workout_response import ClientWorkoutResponse
from .company_details import CompanyDetails
from .connected_source_client_facing import ConnectedSourceClientFacing
from .connection_status import ConnectionStatus
from .connection_status_state import ConnectionStatusState
from .consent import Consent
from .consent_type import ConsentType
from .contraceptive_entry import ContraceptiveEntry
from .contraceptive_entry_type import ContraceptiveEntryType
from .date_part_expr import DatePartExpr
from .date_part_expr_arg import DatePartExprArg
from .date_part_expr_date_part import DatePartExprDatePart
from .date_trunc_expr import DateTruncExpr
from .date_trunc_expr_arg import DateTruncExprArg
from .day_slots import DaySlots
from .delegated_flow_type import DelegatedFlowType
from .demo_connection_status import DemoConnectionStatus
from .demo_providers import DemoProviders
from .detected_deviation_entry import DetectedDeviationEntry
from .detected_deviation_entry_deviation import DetectedDeviationEntryDeviation
from .device_v_2_in_db import DeviceV2InDb
from .email_providers import EmailProviders
from .energy import Energy
from .event_destination_preferences import EventDestinationPreferences
from .event_destination_preferences_enabled_item import EventDestinationPreferencesEnabledItem
from .event_destination_preferences_preferred import EventDestinationPreferencesPreferred
from .failure_type import FailureType
from .fallback_birth_date import FallbackBirthDate
from .fallback_time_zone import FallbackTimeZone
from .fats import Fats
from .gender import Gender
from .get_markers_response import GetMarkersResponse
from .get_orders_response import GetOrdersResponse
from .group_key_column_expr import GroupKeyColumnExpr
from .group_key_column_expr_group_key import GroupKeyColumnExprGroupKey
from .grouped_blood_oxygen import GroupedBloodOxygen
from .grouped_blood_oxygen_response import GroupedBloodOxygenResponse
from .grouped_blood_pressure import GroupedBloodPressure
from .grouped_blood_pressure_response import GroupedBloodPressureResponse
from .grouped_body_fat import GroupedBodyFat
from .grouped_body_fat_response import GroupedBodyFatResponse
from .grouped_body_weight import GroupedBodyWeight
from .grouped_body_weight_response import GroupedBodyWeightResponse
from .grouped_caffeine import GroupedCaffeine
from .grouped_caffeine_response import GroupedCaffeineResponse
from .grouped_calories_active import GroupedCaloriesActive
from .grouped_calories_active_response import GroupedCaloriesActiveResponse
from .grouped_calories_basal import GroupedCaloriesBasal
from .grouped_calories_basal_response import GroupedCaloriesBasalResponse
from .grouped_cholesterol import GroupedCholesterol
from .grouped_cholesterol_response import GroupedCholesterolResponse
from .grouped_distance import GroupedDistance
from .grouped_distance_response import GroupedDistanceResponse
from .grouped_electrocardiogram_voltage import GroupedElectrocardiogramVoltage
from .grouped_electrocardiogram_voltage_response import GroupedElectrocardiogramVoltageResponse
from .grouped_floors_climbed import GroupedFloorsClimbed
from .grouped_floors_climbed_response import GroupedFloorsClimbedResponse
from .grouped_glucose import GroupedGlucose
from .grouped_glucose_response import GroupedGlucoseResponse
from .grouped_heart_rate import GroupedHeartRate
from .grouped_heart_rate_response import GroupedHeartRateResponse
from .grouped_hrv import GroupedHrv
from .grouped_hrv_response import GroupedHrvResponse
from .grouped_hypnogram import GroupedHypnogram
from .grouped_hypnogram_response import GroupedHypnogramResponse
from .grouped_ige import GroupedIge
from .grouped_ige_response import GroupedIgeResponse
from .grouped_igg import GroupedIgg
from .grouped_igg_response import GroupedIggResponse
from .grouped_mindfulness_minutes import GroupedMindfulnessMinutes
from .grouped_mindfulness_minutes_response import GroupedMindfulnessMinutesResponse
from .grouped_respiratory_rate import GroupedRespiratoryRate
from .grouped_respiratory_rate_response import GroupedRespiratoryRateResponse
from .grouped_steps import GroupedSteps
from .grouped_steps_response import GroupedStepsResponse
from .grouped_stress_level import GroupedStressLevel
from .grouped_stress_level_response import GroupedStressLevelResponse
from .grouped_vo_2_max import GroupedVo2Max
from .grouped_vo_2_max_response import GroupedVo2MaxResponse
from .grouped_water import GroupedWater
from .grouped_water_response import GroupedWaterResponse
from .guarantor_details import GuarantorDetails
from .health_insurance_create_request import HealthInsuranceCreateRequest
from .health_insurance_create_request_back_image import HealthInsuranceCreateRequestBackImage
from .health_insurance_create_request_front_image import HealthInsuranceCreateRequestFrontImage
from .health_insurance_create_request_patient_signature_image import HealthInsuranceCreateRequestPatientSignatureImage
from .historical_pull_status import HistoricalPullStatus
from .historical_pull_timeline import HistoricalPullTimeline
from .home_pregnancy_test_entry import HomePregnancyTestEntry
from .home_pregnancy_test_entry_test_result import HomePregnancyTestEntryTestResult
from .home_progesterone_test_entry import HomeProgesteroneTestEntry
from .home_progesterone_test_entry_test_result import HomeProgesteroneTestEntryTestResult
from .http_validation_error import HttpValidationError
from .index_column_expr import IndexColumnExpr
from .index_column_expr_index import IndexColumnExprIndex
from .intermenstrual_bleeding_entry import IntermenstrualBleedingEntry
from .jpeg import Jpeg
from .lab_location_metadata import LabLocationMetadata
from .lab_results_metadata import LabResultsMetadata
from .lab_results_raw import LabResultsRaw
from .lab_results_raw_results import LabResultsRawResults
from .lab_test_collection_method import LabTestCollectionMethod
from .lab_test_sample_type import LabTestSampleType
from .lab_test_status import LabTestStatus
from .labs import Labs
from .last_attempt import LastAttempt
from .libre_config import LibreConfig
from .link_token_exchange_response import LinkTokenExchangeResponse
from .lng_lat import LngLat
from .macros import Macros
from .manual_providers import ManualProviders
from .marker_type import MarkerType
from .meal_in_db_base_client_facing_source import MealInDbBaseClientFacingSource
from .menstrual_cycle import MenstrualCycle
from .menstrual_cycle_response import MenstrualCycleResponse
from .menstrual_flow_entry import MenstrualFlowEntry
from .menstrual_flow_entry_flow import MenstrualFlowEntryFlow
from .metrics_result import MetricsResult
from .micros import Micros
from .minerals import Minerals
from .missing_biomarker_result import MissingBiomarkerResult
from .o_auth_providers import OAuthProviders
from .order_set_request import OrderSetRequest
from .order_status import OrderStatus
from .order_top_level_status import OrderTopLevelStatus
from .ovulation_test_entry import OvulationTestEntry
from .ovulation_test_entry_test_result import OvulationTestEntryTestResult
from .paginated_users_response import PaginatedUsersResponse
from .parent_biomarker_data import ParentBiomarkerData
from .password_providers import PasswordProviders
from .patient_address_compatible import PatientAddressCompatible
from .patient_details import PatientDetails
from .period import Period
from .period_unit import PeriodUnit
from .person_details_output import PersonDetailsOutput
from .phlebotomy_area_info import PhlebotomyAreaInfo
from .phlebotomy_provider_info import PhlebotomyProviderInfo
from .physician_create_request import PhysicianCreateRequest
from .physician_create_request_base import PhysicianCreateRequestBase
from .physician_create_request_signature_image import PhysicianCreateRequestSignatureImage
from .placeholder import Placeholder
from .png import Png
from .post_order_response import PostOrderResponse
from .profile_in_db import ProfileInDb
from .provider_link_response import ProviderLinkResponse
from .provider_link_response_state import ProviderLinkResponseState
from .provider_mfa_request import ProviderMfaRequest
from .provider_mfa_request_method import ProviderMfaRequestMethod
from .providers import Providers
from .psc_area_info import PscAreaInfo
from .psc_area_info_details import PscAreaInfoDetails
from .psc_info import PscInfo
from .query_config import QueryConfig
from .query_config_provider_priority_overrides_item import QueryConfigProviderPriorityOverridesItem
from .query_config_week_starts_on import QueryConfigWeekStartsOn
from .query_instruction import QueryInstruction
from .query_instruction_group_by_item import QueryInstructionGroupByItem
from .query_instruction_select_item import QueryInstructionSelectItem
from .question import Question
from .question_type import QuestionType
from .raw_activity import RawActivity
from .raw_body import RawBody
from .raw_devices import RawDevices
from .raw_profile import RawProfile
from .raw_sleep import RawSleep
from .raw_workout import RawWorkout
from .region import Region
from .relative_timeframe import RelativeTimeframe
from .resource_availability import ResourceAvailability
from .responsible_relationship import ResponsibleRelationship
from .result_type import ResultType
from .scope_requirements_grants import ScopeRequirementsGrants
from .scope_requirements_str import ScopeRequirementsStr
from .select import Select
from .sexual_activity_entry import SexualActivityEntry
from .shipping_address import ShippingAddress
from .single_historical_pull_statistics import SingleHistoricalPullStatistics
from .single_provider_historical_pull_response import SingleProviderHistoricalPullResponse
from .single_resource_statistics import SingleResourceStatistics
from .single_user_historical_pull_response import SingleUserHistoricalPullResponse
from .single_user_resource_response import SingleUserResourceResponse
from .sleep_column_expr import SleepColumnExpr
from .sleep_column_expr_sleep import SleepColumnExprSleep
from .sleep_cycle import SleepCycle
from .sleep_cycle_source_provider import SleepCycleSourceProvider
from .sleep_cycle_source_type import SleepCycleSourceType
from .sleep_score_value_macro_expr import SleepScoreValueMacroExpr
from .sleep_summary_state import SleepSummaryState
from .sleep_type import SleepType
from .sleep_v_2_in_db import SleepV2InDb
from .source import Source
from .source_auth_type import SourceAuthType
from .source_link import SourceLink
from .source_type import SourceType
from .team_config import TeamConfig
from .time_slot import TimeSlot
from .timeseries_metric_point import TimeseriesMetricPoint
from .timeseries_resource import TimeseriesResource
from .trace_elements import TraceElements
from .unrecognized_value_macro_expr import UnrecognizedValueMacroExpr
from .us_address import UsAddress
from .user_historical_pulls_response import UserHistoricalPullsResponse
from .user_info import UserInfo
from .user_refresh_error_response import UserRefreshErrorResponse
from .user_refresh_success_response import UserRefreshSuccessResponse
from .user_resources_response import UserResourcesResponse
from .user_sign_in_token_response import UserSignInTokenResponse
from .user_success_response import UserSuccessResponse
from .validation_error import ValidationError
from .validation_error_loc_item import ValidationErrorLocItem
from .vital_core_schemas_db_schemas_lab_test_health_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestHealthInsurancePersonDetails,
)
from .vital_core_schemas_db_schemas_lab_test_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails,
)
from .vital_sleep_stage import VitalSleepStage
from .vital_token_created_response import VitalTokenCreatedResponse
from .vitamins import Vitamins
from .workout_column_expr import WorkoutColumnExpr
from .workout_column_expr_workout import WorkoutColumnExprWorkout
from .workout_v_2_in_db import WorkoutV2InDb

__all__ = [
    "ActivityColumnExpr",
    "ActivityColumnExprActivity",
    "ActivityV2InDb",
    "AddOnOrder",
    "Address",
    "AggregateExpr",
    "AggregateExprArg",
    "AggregateExprFunc",
    "AllowedRadius",
    "Answer",
    "AoE",
    "AoEAnswer",
    "AppointmentAvailabilitySlots",
    "AppointmentBookingRequest",
    "AppointmentEventStatus",
    "AppointmentLocation",
    "AppointmentProvider",
    "AppointmentPscLabs",
    "AppointmentRescheduleRequest",
    "AppointmentServiceType",
    "AppointmentStatus",
    "AppointmentType",
    "AreaInfo",
    "AttemptStatus",
    "AuthType",
    "Availability",
    "BasalBodyTemperatureEntry",
    "Billing",
    "BiomarkerResult",
    "BodyColumnExpr",
    "BodyColumnExprBody",
    "BodyV2InDb",
    "CervicalMucusEntry",
    "CervicalMucusEntryQuality",
    "ClientActivityResponse",
    "ClientBodyResponse",
    "ClientFacingActivity",
    "ClientFacingApiKey",
    "ClientFacingAppointment",
    "ClientFacingAppointmentCancellationReason",
    "ClientFacingAppointmentEvent",
    "ClientFacingAtHomePhlebotomyOrder",
    "ClientFacingAtHomePhlebotomyOrderDetails",
    "ClientFacingBloodOxygenTimeseries",
    "ClientFacingBloodPressureTimeseries",
    "ClientFacingBody",
    "ClientFacingBodyFatTimeseries",
    "ClientFacingBodyTemperatureDeltaSample",
    "ClientFacingBodyTemperatureDeltaSampleSensorLocation",
    "ClientFacingBodyTemperatureSample",
    "ClientFacingBodyTemperatureSampleSensorLocation",
    "ClientFacingBodyWeightTimeseries",
    "ClientFacingCaffeineTimeseries",
    "ClientFacingCaloriesActiveTimeseries",
    "ClientFacingCaloriesBasalTimeseries",
    "ClientFacingCarbohydratesSample",
    "ClientFacingCholesterolTimeseries",
    "ClientFacingDiagnosisInformation",
    "ClientFacingDistanceTimeseries",
    "ClientFacingElectrocardiogramVoltageTimeseries",
    "ClientFacingFloorsClimbedTimeseries",
    "ClientFacingFood",
    "ClientFacingGlucoseTimeseries",
    "ClientFacingGroupedTimeseriesResponseClientFacingBodyTemperatureDeltaSample",
    "ClientFacingGroupedTimeseriesResponseClientFacingBodyTemperatureSample",
    "ClientFacingGroupedTimeseriesResponseClientFacingCarbohydratesSample",
    "ClientFacingGroupedTimeseriesResponseClientFacingInsulinInjectionSample",
    "ClientFacingGroupedTimeseriesResponseClientFacingNoteSample",
    "ClientFacingGroupedTimeseriesResponseClientFacingWorkoutDurationSample",
    "ClientFacingHeartRate",
    "ClientFacingHeartRateTimeseries",
    "ClientFacingHrvTimeseries",
    "ClientFacingHypnogramTimeseries",
    "ClientFacingIgeTimeseries",
    "ClientFacingIggTimeseries",
    "ClientFacingInsulinInjectionSample",
    "ClientFacingInsulinInjectionSampleType",
    "ClientFacingInsurance",
    "ClientFacingLab",
    "ClientFacingLabLocation",
    "ClientFacingLabTest",
    "ClientFacingLabs",
    "ClientFacingLoinc",
    "ClientFacingMarker",
    "ClientFacingMarkerComplete",
    "ClientFacingMealResponse",
    "ClientFacingMindfulnessMinutesTimeseries",
    "ClientFacingNoteSample",
    "ClientFacingNoteSampleTagsItem",
    "ClientFacingOrder",
    "ClientFacingOrderDetails",
    "ClientFacingOrderDetails_AtHomePhlebotomy",
    "ClientFacingOrderDetails_Testkit",
    "ClientFacingOrderDetails_WalkInTest",
    "ClientFacingOrderEvent",
    "ClientFacingPatientDetailsCompatible",
    "ClientFacingPayorSearchResponse",
    "ClientFacingPhysician",
    "ClientFacingProfile",
    "ClientFacingProvider",
    "ClientFacingProviderDetailed",
    "ClientFacingProviderWithStatus",
    "ClientFacingResource",
    "ClientFacingRespiratoryRateTimeseries",
    "ClientFacingResult",
    "ClientFacingSampleGroupingKeys",
    "ClientFacingShipment",
    "ClientFacingSleep",
    "ClientFacingSleepStream",
    "ClientFacingSource",
    "ClientFacingSport",
    "ClientFacingStepsTimeseries",
    "ClientFacingStream",
    "ClientFacingStreamAltitude",
    "ClientFacingStreamCadence",
    "ClientFacingStreamDistance",
    "ClientFacingStreamHeartrate",
    "ClientFacingStreamLat",
    "ClientFacingStreamLng",
    "ClientFacingStreamPower",
    "ClientFacingStreamResistance",
    "ClientFacingStreamTemperature",
    "ClientFacingStreamVelocitySmooth",
    "ClientFacingStressLevelTimeseries",
    "ClientFacingTeam",
    "ClientFacingTestKitOrderDetails",
    "ClientFacingTestkitOrder",
    "ClientFacingTimeseriesGroupClientFacingBodyTemperatureDeltaSample",
    "ClientFacingTimeseriesGroupClientFacingBodyTemperatureSample",
    "ClientFacingTimeseriesGroupClientFacingCarbohydratesSample",
    "ClientFacingTimeseriesGroupClientFacingInsulinInjectionSample",
    "ClientFacingTimeseriesGroupClientFacingNoteSample",
    "ClientFacingTimeseriesGroupClientFacingWorkoutDurationSample",
    "ClientFacingUser",
    "ClientFacingUserKey",
    "ClientFacingVo2MaxTimeseries",
    "ClientFacingWalkInOrderDetails",
    "ClientFacingWalkInTestOrder",
    "ClientFacingWaterTimeseries",
    "ClientFacingWorkout",
    "ClientFacingWorkoutDurationSample",
    "ClientFacingWorkoutDurationSampleIntensity",
    "ClientSleepCycleResponse",
    "ClientSleepResponse",
    "ClientUserIdConflict",
    "ClientWorkoutResponse",
    "CompanyDetails",
    "ConnectedSourceClientFacing",
    "ConnectionStatus",
    "ConnectionStatusState",
    "Consent",
    "ConsentType",
    "ContraceptiveEntry",
    "ContraceptiveEntryType",
    "DatePartExpr",
    "DatePartExprArg",
    "DatePartExprDatePart",
    "DateTruncExpr",
    "DateTruncExprArg",
    "DaySlots",
    "DelegatedFlowType",
    "DemoConnectionStatus",
    "DemoProviders",
    "DetectedDeviationEntry",
    "DetectedDeviationEntryDeviation",
    "DeviceV2InDb",
    "EmailProviders",
    "Energy",
    "EventDestinationPreferences",
    "EventDestinationPreferencesEnabledItem",
    "EventDestinationPreferencesPreferred",
    "FailureType",
    "FallbackBirthDate",
    "FallbackTimeZone",
    "Fats",
    "Gender",
    "GetMarkersResponse",
    "GetOrdersResponse",
    "GroupKeyColumnExpr",
    "GroupKeyColumnExprGroupKey",
    "GroupedBloodOxygen",
    "GroupedBloodOxygenResponse",
    "GroupedBloodPressure",
    "GroupedBloodPressureResponse",
    "GroupedBodyFat",
    "GroupedBodyFatResponse",
    "GroupedBodyWeight",
    "GroupedBodyWeightResponse",
    "GroupedCaffeine",
    "GroupedCaffeineResponse",
    "GroupedCaloriesActive",
    "GroupedCaloriesActiveResponse",
    "GroupedCaloriesBasal",
    "GroupedCaloriesBasalResponse",
    "GroupedCholesterol",
    "GroupedCholesterolResponse",
    "GroupedDistance",
    "GroupedDistanceResponse",
    "GroupedElectrocardiogramVoltage",
    "GroupedElectrocardiogramVoltageResponse",
    "GroupedFloorsClimbed",
    "GroupedFloorsClimbedResponse",
    "GroupedGlucose",
    "GroupedGlucoseResponse",
    "GroupedHeartRate",
    "GroupedHeartRateResponse",
    "GroupedHrv",
    "GroupedHrvResponse",
    "GroupedHypnogram",
    "GroupedHypnogramResponse",
    "GroupedIge",
    "GroupedIgeResponse",
    "GroupedIgg",
    "GroupedIggResponse",
    "GroupedMindfulnessMinutes",
    "GroupedMindfulnessMinutesResponse",
    "GroupedRespiratoryRate",
    "GroupedRespiratoryRateResponse",
    "GroupedSteps",
    "GroupedStepsResponse",
    "GroupedStressLevel",
    "GroupedStressLevelResponse",
    "GroupedVo2Max",
    "GroupedVo2MaxResponse",
    "GroupedWater",
    "GroupedWaterResponse",
    "GuarantorDetails",
    "HealthInsuranceCreateRequest",
    "HealthInsuranceCreateRequestBackImage",
    "HealthInsuranceCreateRequestFrontImage",
    "HealthInsuranceCreateRequestPatientSignatureImage",
    "HistoricalPullStatus",
    "HistoricalPullTimeline",
    "HomePregnancyTestEntry",
    "HomePregnancyTestEntryTestResult",
    "HomeProgesteroneTestEntry",
    "HomeProgesteroneTestEntryTestResult",
    "HttpValidationError",
    "IndexColumnExpr",
    "IndexColumnExprIndex",
    "IntermenstrualBleedingEntry",
    "Jpeg",
    "LabLocationMetadata",
    "LabResultsMetadata",
    "LabResultsRaw",
    "LabResultsRawResults",
    "LabTestCollectionMethod",
    "LabTestSampleType",
    "LabTestStatus",
    "Labs",
    "LastAttempt",
    "LibreConfig",
    "LinkTokenExchangeResponse",
    "LngLat",
    "Macros",
    "ManualProviders",
    "MarkerType",
    "MealInDbBaseClientFacingSource",
    "MenstrualCycle",
    "MenstrualCycleResponse",
    "MenstrualFlowEntry",
    "MenstrualFlowEntryFlow",
    "MetricsResult",
    "Micros",
    "Minerals",
    "MissingBiomarkerResult",
    "OAuthProviders",
    "OrderSetRequest",
    "OrderStatus",
    "OrderTopLevelStatus",
    "OvulationTestEntry",
    "OvulationTestEntryTestResult",
    "PaginatedUsersResponse",
    "ParentBiomarkerData",
    "PasswordProviders",
    "PatientAddressCompatible",
    "PatientDetails",
    "Period",
    "PeriodUnit",
    "PersonDetailsOutput",
    "PhlebotomyAreaInfo",
    "PhlebotomyProviderInfo",
    "PhysicianCreateRequest",
    "PhysicianCreateRequestBase",
    "PhysicianCreateRequestSignatureImage",
    "Placeholder",
    "Png",
    "PostOrderResponse",
    "ProfileInDb",
    "ProviderLinkResponse",
    "ProviderLinkResponseState",
    "ProviderMfaRequest",
    "ProviderMfaRequestMethod",
    "Providers",
    "PscAreaInfo",
    "PscAreaInfoDetails",
    "PscInfo",
    "QueryConfig",
    "QueryConfigProviderPriorityOverridesItem",
    "QueryConfigWeekStartsOn",
    "QueryInstruction",
    "QueryInstructionGroupByItem",
    "QueryInstructionSelectItem",
    "Question",
    "QuestionType",
    "RawActivity",
    "RawBody",
    "RawDevices",
    "RawProfile",
    "RawSleep",
    "RawWorkout",
    "Region",
    "RelativeTimeframe",
    "ResourceAvailability",
    "ResponsibleRelationship",
    "ResultType",
    "ScopeRequirementsGrants",
    "ScopeRequirementsStr",
    "Select",
    "SexualActivityEntry",
    "ShippingAddress",
    "SingleHistoricalPullStatistics",
    "SingleProviderHistoricalPullResponse",
    "SingleResourceStatistics",
    "SingleUserHistoricalPullResponse",
    "SingleUserResourceResponse",
    "SleepColumnExpr",
    "SleepColumnExprSleep",
    "SleepCycle",
    "SleepCycleSourceProvider",
    "SleepCycleSourceType",
    "SleepScoreValueMacroExpr",
    "SleepSummaryState",
    "SleepType",
    "SleepV2InDb",
    "Source",
    "SourceAuthType",
    "SourceLink",
    "SourceType",
    "TeamConfig",
    "TimeSlot",
    "TimeseriesMetricPoint",
    "TimeseriesResource",
    "TraceElements",
    "UnrecognizedValueMacroExpr",
    "UsAddress",
    "UserHistoricalPullsResponse",
    "UserInfo",
    "UserRefreshErrorResponse",
    "UserRefreshSuccessResponse",
    "UserResourcesResponse",
    "UserSignInTokenResponse",
    "UserSuccessResponse",
    "ValidationError",
    "ValidationErrorLocItem",
    "VitalCoreSchemasDbSchemasLabTestHealthInsurancePersonDetails",
    "VitalCoreSchemasDbSchemasLabTestInsurancePersonDetails",
    "VitalSleepStage",
    "VitalTokenCreatedResponse",
    "Vitamins",
    "WorkoutColumnExpr",
    "WorkoutColumnExprWorkout",
    "WorkoutV2InDb",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class Minerals(str, enum.Enum):
    SODIUM = "sodium"
    POTASSIUM = "potassium"
    CALCIUM = "calcium"
    PHOSPHORUS = "phosphorus"
    MAGNESIUM = "magnesium"
    IRON = "iron"
    ZINC = "zinc"
    FLUORIDE = "fluoride"
    CHLORIDE = "chloride"
    BORON = "boron"
    COBALT = "cobalt"
    NICKEL = "nickel"
    SILICON = "silicon"
    VANADIUM = "vanadium"
    LITHIUM = "lithium"
    RUBIDIUM = "rubidium"
    STRONTIUM = "strontium"
    ALUMINUM = "aluminum"
    ARSENIC = "arsenic"
    BARIUM = "barium"
    BERYLLIUM = "beryllium"
    BISMUTH = "bismuth"
    CADMIUM = "cadmium"
    CESIUM = "cesium"
    GERMANIUM = "germanium"
    GOLD = "gold"
    LEAD = "lead"
    MERCURY = "mercury"
    PALLADIUM = "palladium"
    PLATINUM = "platinum"
    SILVER = "silver"
    THALLIUM = "thallium"
    THORIUM = "thorium"
    TIN = "tin"
    TITANIUM = "titanium"
    TUNGSTEN = "tungsten"
    URANIUM = "uranium"
    ZIRCONIUM = "zirconium"

    def visit(
        self,
        sodium: typing.Callable[[], T_Result],
        potassium: typing.Callable[[], T_Result],
        calcium: typing.Callable[[], T_Result],
        phosphorus: typing.Callable[[], T_Result],
        magnesium: typing.Callable[[], T_Result],
        iron: typing.Callable[[], T_Result],
        zinc: typing.Callable[[], T_Result],
        fluoride: typing.Callable[[], T_Result],
        chloride: typing.Callable[[], T_Result],
        boron: typing.Callable[[], T_Result],
        cobalt: typing.Callable[[], T_Result],
        nickel: typing.Callable[[], T_Result],
        silicon: typing.Callable[[], T_Result],
        vanadium: typing.Callable[[], T_Result],
        lithium: typing.Callable[[], T_Result],
        rubidium: typing.Callable[[], T_Result],
        strontium: typing.Callable[[], T_Result],
        aluminum: typing.Callable[[], T_Result],
        arsenic: typing.Callable[[], T_Result],
        barium: typing.Callable[[], T_Result],
        beryllium: typing.Callable[[], T_Result],
        bismuth: typing.Callable[[], T_Result],
        cadmium: typing.Callable[[], T_Result],
        cesium: typing.Callable[[], T_Result],
        germanium: typing.Callable[[], T_Result],
        gold: typing.Callable[[], T_Result],
        lead: typing.Callable[[], T_Result],
        mercury: typing.Callable[[], T_Result],
        palladium: typing.Callable[[], T_Result],
        platinum: typing.Callable[[], T_Result],
        silver: typing.Callable[[], T_Result],
        thallium: typing.Callable[[], T_Result],
        thorium: typing.Callable[[], T_Result],
        tin: typing.Callable[[], T_Result],
        titanium: typing.Callable[[], T_Result],
        tungsten: typing.Callable[[], T_Result],
        uranium: typing.Callable[[], T_Result],
        zirconium: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is Minerals.SODIUM:
            return sodium()
        if self is Minerals.POTASSIUM:
            return potassium()
        if self is Minerals.CALCIUM:
            return calcium()
        if self is Minerals.PHOSPHORUS:
            return phosphorus()
        if self is Minerals.MAGNESIUM:
            return magnesium()
        if self is Minerals.IRON:
            return iron()
        if self is Minerals.ZINC:
            return zinc()
        if self is Minerals.FLUORIDE:
            return fluoride()
        if self is Minerals.CHLORIDE:
            return chloride()
        if self is Minerals.BORON:
            return boron()
        if self is Minerals.COBALT:
            return cobalt()
        if self is Minerals.NICKEL:
            return nickel()
        if self is Minerals.SILICON:
            return silicon()
        if self is Minerals.VANADIUM:
            return vanadium()
        if self is Minerals.LITHIUM:
            return lithium()
        if self is Minerals.RUBIDIUM:
            return rubidium()
        if self is Minerals.STRONTIUM:
            return strontium()
        if self is Minerals.ALUMINUM:
            return aluminum()
        if self is Minerals.ARSENIC:
            return arsenic()
        if self is Minerals.BARIUM:
            return barium()
        if self is Minerals.BERYLLIUM:
            return beryllium()
        if self is Minerals.BISMUTH:
            return bismuth()
        if self is Minerals.CADMIUM:
            return cadmium()
        if self is Minerals.CESIUM:
            return cesium()
        if self is Minerals.GERMANIUM:
            return germanium()
        if self is Minerals.GOLD:
            return gold()
        if self is Minerals.LEAD:
            return lead()
        if self is Minerals.MERCURY:
            return mercury()
        if self is Minerals.PALLADIUM:
            return palladium()
        if self is Minerals.PLATINUM:
            return platinum()
        if self is Minerals.SILVER:
            return silver()
        if self is Minerals.THALLIUM:
            return thallium()
        if self is Minerals.THORIUM:
            return thorium()
        if self is Minerals.TIN:
            return tin()
        if self is Minerals.TITANIUM:
            return titanium()
        if self is Minerals.TUNGSTEN:
            return tungsten()
        if self is Minerals.URANIUM:
            return uranium()
        if self is Minerals.ZIRCONIUM:
            return zirconium()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
from .phlebotomy_area_info import PhlebotomyAreaInfo
import typing
from .psc_area_info import PscAreaInfo
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class AreaInfo(UniversalBaseModel):
    zip_code: str
    phlebotomy: PhlebotomyAreaInfo
    central_labs: typing.Dict[str, PscAreaInfo]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .health_insurance_create_request_front_image import HealthInsuranceCreateRequestFrontImage
import pydantic
from .health_insurance_create_request_back_image import HealthInsuranceCreateRequestBackImage
from .health_insurance_create_request_patient_signature_image import HealthInsuranceCreateRequestPatientSignatureImage
from .responsible_relationship import ResponsibleRelationship
from .vital_core_schemas_db_schemas_lab_test_health_insurance_person_details import (
    VitalCoreSchemasDbSchemasLabTestHealthInsurancePersonDetails,
)
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class HealthInsuranceCreateRequest(UniversalBaseModel):
    front_image: typing.Optional[HealthInsuranceCreateRequestFrontImage] = pydantic.Field(default=None)
    """
    An image of the front of the patient insurance card.
    """

    back_image: typing.Optional[HealthInsuranceCreateRequestBackImage] = pydantic.Field(default=None)
    """
    An image of the back of the patient insurance card.
    """

    patient_signature_image: typing.Optional[HealthInsuranceCreateRequestPatientSignatureImage] = pydantic.Field(
        default=None
    )
    """
    An image of the patient signature for health insurance billing.
    """

    subjective: typing.Optional[str] = pydantic.Field(default=None)
    """
    Textual description of what are the patient symptoms and attempted treatments.
    """

    assessment_plan: typing.Optional[str] = pydantic.Field(default=None)
    """
    Textual description of what are the physician assessments and testing plans.
    """

    payor_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier representing a specific Health Insurance.
    """

    insurance_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Insurance unique number assigned to a patient, usually present on the insurance card.
    """

    responsible_relationship: typing.Optional[ResponsibleRelationship] = pydantic.Field(default=None)
    """
    Relationship between the patient and the insurance contractor. Values can be (Self, Spouse, Other Relationship).
    """

    responsible_details: typing.Optional[VitalCoreSchemasDbSchemasLabTestHealthInsurancePersonDetails] = pydantic.Field(
        default=None
    )
    """
    Responsible details when the value of responsible_relationship is not 'Self'.
    """

    diagnosis_codes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Diagnosis codes for insurance billing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingLabs(str, enum.Enum):
    QUEST = "quest"
    LABCORP = "labcorp"
    BIOREFERENCE = "bioreference"

    def visit(
        self,
        quest: typing.Callable[[], T_Result],
        labcorp: typing.Callable[[], T_Result],
        bioreference: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingLabs.QUEST:
            return quest()
        if self is ClientFacingLabs.LABCORP:
            return labcorp()
        if self is ClientFacingLabs.BIOREFERENCE:
            return bioreference()



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClientFacingBodyTemperatureDeltaSampleSensorLocation(str, enum.Enum):
    ARMPIT = "armpit"
    BODY = "body"
    EAR = "ear"
    FINGER = "finger"
    GASTROINTESTINAL = "gastrointestinal"
    MOUTH = "mouth"
    RECTUM = "rectum"
    TOE = "toe"
    EARDRUM = "eardrum"
    TEMPORAL_ARTERY = "temporal_artery"
    FOREHEAD = "forehead"
    WRIST = "wrist"

    def visit(
        self,
        armpit: typing.Callable[[], T_Result],
        body: typing.Callable[[], T_Result],
        ear: typing.Callable[[], T_Result],
        finger: typing.Callable[[], T_Result],
        gastrointestinal: typing.Callable[[], T_Result],
        mouth: typing.Callable[[], T_Result],
        rectum: typing.Callable[[], T_Result],
        toe: typing.Callable[[], T_Result],
        eardrum: typing.Callable[[], T_Result],
        temporal_artery: typing.Callable[[], T_Result],
        forehead: typing.Callable[[], T_Result],
        wrist: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.ARMPIT:
            return armpit()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.BODY:
            return body()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.EAR:
            return ear()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.FINGER:
            return finger()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.GASTROINTESTINAL:
            return gastrointestinal()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.MOUTH:
            return mouth()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.RECTUM:
            return rectum()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.TOE:
            return toe()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.EARDRUM:
            return eardrum()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.TEMPORAL_ARTERY:
            return temporal_artery()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.FOREHEAD:
            return forehead()
        if self is ClientFacingBodyTemperatureDeltaSampleSensorLocation.WRIST:
            return wrist()



# This file was auto-generated by Fern from our API Definition.

from ..core.pydantic_utilities import UniversalBaseModel
import typing
from .query_instruction_select_item import QueryInstructionSelectItem
from .query_instruction_group_by_item import QueryInstructionGroupByItem
from ..core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic


class QueryInstruction(UniversalBaseModel):
    select: typing.List[QueryInstructionSelectItem]
    group_by: typing.Optional[typing.List[QueryInstructionGroupByItem]] = None
    split_by_source: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
